import os
import shutil
import site
from pathlib import Path
from subprocess import run
import torch

from torch2hpvm import BinDataset, ModelExporter
from torch.nn import Module

site.addsitedir(os.path.dirname(__file__))
import dnn

benchmarks = [
    (dnn.LeNet, 1, 28, 5000, "lenet_mnist"),
    (dnn.AlexNet, 3, 32, 5000, "alexnet_cifar10"),
    (dnn.AlexNet2, 3, 32, 5000, "alexnet2_cifar10"),
    (dnn.AlexNetImageNet, 3, 224, 500, "alexnet_imagenet"),
    (dnn.MobileNet, 3, 32, 5000, "mobilenet_cifar10"),
    (dnn.ResNet18, 3, 32, 5000, "resnet18_cifar10"),
    (dnn.ResNet50, 3, 224, 100, "resnet50_imagenet"),
    (dnn.VGG16Cifar10, 3, 32, 5000, "vgg16_cifar10"),
    (dnn.VGG16Cifar100, 3, 32, 5000, "vgg16_cifar100"),
    (dnn.VGG16ImageNet, 3, 224, 100, "vgg16_imagenet"),
]
self_folder = Path(__file__).parent
for model_cls, nch, img_size, batch_size, pathname in benchmarks:
    codegen_dir = Path(f"/tmp/{pathname}")
    print(f"Generating {pathname} to {codegen_dir}")
    if codegen_dir.exists():
        shutil.rmtree(codegen_dir)

    params = self_folder / "../model_params" / pathname
    dataset_shape = 5000, nch, img_size, img_size
    bin_tuneset = BinDataset(
        params / "tune_input.bin", params / "tune_labels.bin", dataset_shape
    )
    bin_testset = BinDataset(
        params / "test_input.bin", params / "test_labels.bin", dataset_shape
    )
    model: Module = model_cls()
    checkpoint = self_folder / "../model_params" / f"{pathname}.pth.tar"
    model.load_state_dict(torch.load(checkpoint.as_posix()))

    exporter = ModelExporter(model, bin_tuneset, bin_testset, codegen_dir)
    exporter.export_all(batch_size=batch_size)

    conf_file = self_folder / "../hpvm-c/benchmarks" / pathname / "data/tuner_confs.txt"
    build_dir = codegen_dir / "build"
    target_binary = build_dir / pathname
    run([
        "approxhpvm.py", str(codegen_dir / ModelExporter.source_file_name), str(target_binary),
        "-d", str(build_dir),
        "-t", "tensor", "--conf-file", str(conf_file)
    ], check=True)
    run([str(target_binary), "test"], check=True)
